@@ -17,12 +17,11 @@ from glue.core.roi import XRangeROI, RectangularROI, CircularROI
 from glue.core.subset import RoiSubsetState, AndState
 from glue import core
 from glue.core.component_id import ComponentID
-from glue.utils.qt import combo_as_string
+from glue.utils.qt import combo_as_string, process_events
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
 from glue.core.state import GlueUnSerializer
 from glue.app.qt.layer_tree_widget import LayerTreeWidget
 from glue.app.qt import GlueApplication
-from glue.tests.helpers import PYSIDE2_INSTALLED  # noqa
 
 from ..data_viewer import ScatterViewer
 
@@ -71,12 +70,12 @@ class TestScatterViewer(object):
         assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
 
         assert viewer_state.x_att is self.data.id['x']
-        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
-        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
+        assert_allclose(viewer_state.x_min, -1.1 - 0.18)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.18)
 
         assert viewer_state.y_att is self.data.id['y']
-        assert_allclose(viewer_state.y_min, 3.2 - 0.015)
-        assert_allclose(viewer_state.y_max, 3.5 + 0.015)
+        assert_allclose(viewer_state.y_min, 3.2 - 0.012)
+        assert_allclose(viewer_state.y_max, 3.5 + 0.012)
 
         assert not viewer_state.x_log
         assert not viewer_state.y_log
@@ -88,12 +87,12 @@ class TestScatterViewer(object):
         viewer_state.y_att = self.data.id['z']
 
         assert viewer_state.x_att is self.data.id['x']
-        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
-        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
+        assert_allclose(viewer_state.x_min, -1.1 - 0.18)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.18)
 
         assert viewer_state.y_att is self.data.id['z']
-        assert_allclose(viewer_state.y_min, -0.5 - 0.15)
-        assert_allclose(viewer_state.y_max, 2.5 + 0.15)
+        assert_allclose(viewer_state.y_min, -0.5 - 0.12)
+        assert_allclose(viewer_state.y_max, 2.5 + 0.12)
 
         assert not viewer_state.x_log
         assert not viewer_state.y_log
@@ -104,21 +103,21 @@ class TestScatterViewer(object):
 
         self.viewer.add_data(self.data)
 
-        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
-        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
+        assert_allclose(viewer_state.x_min, -1.1 - 0.18)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.18)
 
         self.viewer.options_widget().button_flip_x.click()
 
-        assert_allclose(viewer_state.x_max, -1.1 - 0.225)
-        assert_allclose(viewer_state.x_min, 3.4 + 0.225)
+        assert_allclose(viewer_state.x_max, -1.1 - 0.18)
+        assert_allclose(viewer_state.x_min, 3.4 + 0.18)
 
-        assert_allclose(viewer_state.y_min, 3.2 - 0.015)
-        assert_allclose(viewer_state.y_max, 3.5 + 0.015)
+        assert_allclose(viewer_state.y_min, 3.2 - 0.012)
+        assert_allclose(viewer_state.y_max, 3.5 + 0.012)
 
         self.viewer.options_widget().button_flip_y.click()
 
-        assert_allclose(viewer_state.y_max, 3.2 - 0.015)
-        assert_allclose(viewer_state.y_min, 3.5 + 0.015)
+        assert_allclose(viewer_state.y_max, 3.2 - 0.012)
+        assert_allclose(viewer_state.y_min, 3.5 + 0.012)
 
     def test_remove_data(self):
         self.viewer.add_data(self.data)
@@ -243,18 +242,15 @@ class TestScatterViewer(object):
 
     def test_density_map(self):
 
+        kwargs = dict(range=[(-5, 5), (-5, 5)], bins=(2, 2))
+
         self.viewer.add_data(self.data)
-
-        density_artist = self.viewer.layers[0].density_artist
-        if hasattr(density_artist, 'histogram2d_helper'):
-            density_artist = density_artist.histogram2d_helper
-
         self.viewer.state.layers[0].points_mode = 'auto'
-        assert len(density_artist._x) == 0
+        assert self.viewer.layers[0].state.compute_density_map(**kwargs).sum() == 0
         self.viewer.state.layers[0].points_mode = 'density'
-        assert len(density_artist._x) == 4
+        assert self.viewer.layers[0].state.compute_density_map(**kwargs).sum() == 4
         self.viewer.state.layers[0].points_mode = 'markers'
-        assert len(density_artist._x) == 0
+        assert self.viewer.layers[0].state.compute_density_map(**kwargs).sum() == 0
 
     def test_density_map_color(self):
 
@@ -268,7 +264,6 @@ class TestScatterViewer(object):
         self.viewer.state.layers[0].points_mode = 'markers'
         self.viewer.state.layers[0].points_mode = 'density'
 
-    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     @pytest.mark.parametrize('protocol', [0, 1])
     def test_session_back_compat(self, protocol):
 
@@ -329,6 +324,8 @@ class TestScatterViewer(object):
         assert viewer3.state.layers[1].visible
         assert not viewer3.state.layers[2].visible
 
+        ga.close()
+
     def test_session_line_back_compat(self):
 
         # Backward-compatibility for v0.11 files in which the line and scatter
@@ -363,6 +360,8 @@ class TestScatterViewer(object):
         assert not viewer1.state.layers[0].markers_visible
         assert viewer1.state.layers[0].line_visible
 
+        ga.close()
+
     def test_save_svg(self, tmpdir):
         # Regression test for a bug in AxesCache that caused SVG saving to
         # fail (because renderer.buffer_rgba did not exist)
@@ -377,12 +376,12 @@ class TestScatterViewer(object):
         self.viewer.add_data(self.data_2d)
 
         assert viewer_state.x_att is self.data_2d.id['a']
-        assert_allclose(viewer_state.x_min, 1 - 0.15)
-        assert_allclose(viewer_state.x_max, 4 + 0.15)
+        assert_allclose(viewer_state.x_min, 1 - 0.12)
+        assert_allclose(viewer_state.x_max, 4 + 0.12)
 
         assert viewer_state.y_att is self.data_2d.id['b']
-        assert_allclose(viewer_state.y_min, 5 - 0.15)
-        assert_allclose(viewer_state.y_max, 8 + 0.15)
+        assert_allclose(viewer_state.y_min, 5 - 0.12)
+        assert_allclose(viewer_state.y_max, 8 + 0.12)
 
         assert self.viewer.layers[0].plot_artist.get_xdata().shape == (4,)
 
@@ -496,6 +495,8 @@ class TestScatterViewer(object):
         self.viewer.add_data(self.data)
         self.viewer.state.x_att = self.data.id['z']
 
+        process_events()
+
         assert visible_xaxis_labels(self.viewer.axes) == ['a', 'b', 'c']
 
         self.session.application.save_session(filename)
@@ -513,6 +514,8 @@ class TestScatterViewer(object):
         assert viewer.state.x_att is dc[0].id['z']
         assert visible_xaxis_labels(self.viewer.axes) == ['a', 'b', 'c']
 
+        ga.close()
+
     def test_enable_disable_components_combo(self):
 
         # Regression test for a bug that caused an error when turning off pixel
@@ -539,8 +542,8 @@ class TestScatterViewer(object):
         # Matplotlib deals with dates by converting them to the number of days
         # since 01-01-0001, so we can check that the limits are correctly
         # converted (and not 100 to 400)
-        assert self.viewer.axes.get_xlim() == (719248.0, 719578.0)
-        assert self.viewer.axes.get_ylim() == (3.2 - 0.015, 3.5 + 0.015)
+        assert self.viewer.axes.get_xlim() == (719251.0, 719575.0)
+        assert self.viewer.axes.get_ylim() == (3.2 - 0.012, 3.5 + 0.012)
 
         # Apply an ROI selection in plotting coordinates
         roi = RectangularROI(xmin=719313, xmax=719513, ymin=3, ymax=4)
@@ -552,8 +555,8 @@ class TestScatterViewer(object):
         # Now do the same with the y axis
         self.viewer.state.y_att = self.data.id['t2']
 
-        assert self.viewer.axes.get_xlim() == (719248.0, 719578.0)
-        assert self.viewer.axes.get_ylim() == (719348.0, 719678.0)
+        assert self.viewer.axes.get_xlim() == (719251.0, 719575.0)
+        assert self.viewer.axes.get_ylim() == (719351.0, 719675.0)
 
         # Apply an ROI selection in plotting coordinates
         edit = self.session.edit_subset_mode
@@ -565,16 +568,16 @@ class TestScatterViewer(object):
         # Make sure that the Qt labels look ok
         self.viewer.state.y_att = self.data.id['y']
         options = self.viewer.options_widget().ui
-        assert options.valuetext_x_min.text() == '1970-03-27'
-        assert options.valuetext_x_max.text() == '1971-02-20'
-        assert options.valuetext_y_min.text() == '3.185'
-        assert options.valuetext_y_max.text() == '3.515'
+        assert options.valuetext_x_min.text() == '1970-03-30'
+        assert options.valuetext_x_max.text() == '1971-02-17'
+        assert options.valuetext_y_min.text() == '3.188'
+        assert options.valuetext_y_max.text() == '3.512'
 
         # Make sure that we can set the xmin/xmax to a string date
-        assert_equal(self.viewer.state.x_min, np.datetime64('1970-03-27', 'D'))
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-03-30', 'D'))
         options.valuetext_x_min.setText('1970-04-14')
         options.valuetext_x_min.editingFinished.emit()
-        assert self.viewer.axes.get_xlim() == (719266.0, 719578.0)
+        assert self.viewer.axes.get_xlim() == (719266.0, 719575.0)
         assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
 
         # Make sure that everything works fine after saving/reloading
@@ -590,6 +593,8 @@ class TestScatterViewer(object):
         assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
 
         assert options.valuetext_x_min.text() == '1970-04-14'
-        assert options.valuetext_x_max.text() == '1971-02-20'
-        assert options.valuetext_y_min.text() == '3.185'
-        assert options.valuetext_y_max.text() == '3.515'
+        assert options.valuetext_x_max.text() == '1971-02-17'
+        assert options.valuetext_y_min.text() == '3.188'
+        assert options.valuetext_y_max.text() == '3.512'
+
+        ga.close()
